/**
 *******************************************************************************
 * @file    tmpm461_gpio.h
 * @brief   This file provides all the functions prototypes for GPIO driver. 
 * @version V2.0.2.2
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __TMPM461_GPIO_H
#define __TMPM461_GPIO_H

#ifdef __cplusplus
extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/
#include "TMPM461.h"
#include "tx04_common.h"


/** @addtogroup TX04_Periph_Driver
  * @{
  */
/** @addtogroup GPIO
  * @{
  */
/** @addtogroup  GPIO_Parameter_Definition
  * @{
  */


/** @brief :The maximum number of the Function Register
  * Note for porting:
  * If functiong register 5 is the maximum number in 
  * all the GPIO port,then define FRMAX (5U)
  */
#define FRMAX  (5U)             /* the max number of Port I/O function register is 5 */


/** @brief: define for function register
  * Note for porting:
  * If the maximum number of the function Register is 5,
  * then you need to define 5 GPIO_FUNC_REG_x ,
  * the value should be increased from 0 to 5
  */
#define GPIO_FUNC_REG_1                 ((uint16_t)0x0000)
#define GPIO_FUNC_REG_2                 ((uint16_t)0x0001)
#define GPIO_FUNC_REG_3                 ((uint16_t)0x0002)
#define GPIO_FUNC_REG_4                 ((uint16_t)0x0003)
#define GPIO_FUNC_REG_5                 ((uint16_t)0x0004)

/** @brief :The GPIO_Port enum
  * Note for porting:
  * the port value order from low to high with '1' step
  * and begin with "0".
  */
    typedef enum {
        GPIO_PA = 0U,
        GPIO_PB = 1U,
        GPIO_PC = 2U,
        GPIO_PD = 3U,
        GPIO_PE = 4U,
        GPIO_PF = 5U,
        GPIO_PG = 6U,
        GPIO_PH = 7U,
        GPIO_PJ = 8U,
        GPIO_PK = 9U,
    } GPIO_Port;

#define IS_GPIO_PORT(param)             ((param) <= GPIO_PK)    /*parameter checking for port number */


#define RESER  (8U-(FRMAX))

    typedef struct {
        __IO uint32_t DATA;
        __IO uint32_t CR;
        __IO uint32_t FR[FRMAX];
        uint32_t RESERVED0[RESER];
        __IO uint32_t OD;
        __IO uint32_t PUP;
        __IO uint32_t PDN;
        uint32_t RESERVED1;
        __IO uint32_t IE;
    } TSB_Port_TypeDef;

    typedef struct {
        uint16_t PinDATA;
        uint16_t PinCR;
        uint16_t PinFR[FRMAX];
        uint16_t PinOD;
        uint16_t PinPUP;
        uint16_t PinPDN;
        uint16_t PinIE;
    } GPIO_RegTypeDef;

    typedef struct {
        uint16_t IOMode;        /* Set the port input or output mode       */
        uint16_t PullUp;        /* Enable or disable Pullup function       */
        uint16_t OpenDrain;     /* Enable or disable open drain function */
        uint16_t PullDown;      /* Enable or disable Pulldown function */
    } GPIO_InitTypeDef;

#define GPIO_INPUT_MODE                 ((uint16_t)0x0000)
#define GPIO_OUTPUT_MODE                ((uint16_t)0x0001)
#define GPIO_IO_MODE_NONE               ((uint16_t)0x0002)
#define IS_GPIO_IO_MODE_STATE(param)    (((param) == GPIO_INPUT_MODE) || \
                                         ((param) == GPIO_OUTPUT_MODE) || \
                                         ((param) == GPIO_IO_MODE_NONE))

#define GPIO_PULLUP_DISABLE             ((uint16_t)0x0000)
#define GPIO_PULLUP_ENABLE              ((uint16_t)0x0001)
#define GPIO_PULLUP_NONE                ((uint16_t)0x0002)
#define IS_GPIO_PULLUP_STATE(param)     (((param) == GPIO_PULLUP_ENABLE) || \
                                         ((param) == GPIO_PULLUP_DISABLE) || \
                                         ((param) == GPIO_PULLUP_NONE))

#define GPIO_PULLDOWN_DISABLE           ((uint16_t)0x0000)
#define GPIO_PULLDOWN_ENABLE            ((uint16_t)0x0001)
#define GPIO_PULLDOWN_NONE              ((uint16_t)0x0002)
#define IS_GPIO_PULLDOWN_STATE(param)   (((param) == GPIO_PULLDOWN_ENABLE) || \
                                         ((param) == GPIO_PULLDOWN_DISABLE) || \
                                         ((param) == GPIO_PULLDOWN_NONE))

#define GPIO_OPEN_DRAIN_DISABLE         ((uint16_t)0x0000)
#define GPIO_OPEN_DRAIN_ENABLE          ((uint16_t)0x0001)
#define GPIO_OPEN_DRAIN_NONE            ((uint16_t)0x0002)
#define IS_GPIO_OPEN_DRAIN_STATE(param) (((param) == GPIO_OPEN_DRAIN_ENABLE) || \
                                         ((param) == GPIO_OPEN_DRAIN_DISABLE) || \
                                         ((param) == GPIO_OPEN_DRAIN_NONE))

#define GPIO_BIT_VALUE_1                ((uint16_t)0x0001)
#define GPIO_BIT_VALUE_0                ((uint16_t)0x0000)

#define IS_GPIO_BIT_VALUE(BitValue)     (((BitValue) == GPIO_BIT_VALUE_1)|| \
                                         ((BitValue) == GPIO_BIT_VALUE_0))

#define GPIO_BIT_0                      ((uint16_t)0x0001)
#define GPIO_BIT_1                      ((uint16_t)0x0002)
#define GPIO_BIT_2                      ((uint16_t)0x0004)
#define GPIO_BIT_3                      ((uint16_t)0x0008)
#define GPIO_BIT_4                      ((uint16_t)0x0010)
#define GPIO_BIT_5                      ((uint16_t)0x0020)
#define GPIO_BIT_6                      ((uint16_t)0x0040)
#define GPIO_BIT_7                      ((uint16_t)0x0080)
#define GPIO_BIT_8                      ((uint16_t)0x0100)
#define GPIO_BIT_9                      ((uint16_t)0x0200)
#define GPIO_BIT_10                     ((uint16_t)0x0400)
#define GPIO_BIT_11                     ((uint16_t)0x0800)
#define GPIO_BIT_12                     ((uint16_t)0x1000)
#define GPIO_BIT_13                     ((uint16_t)0x2000)
#define GPIO_BIT_14                     ((uint16_t)0x4000)
#define GPIO_BIT_15                     ((uint16_t)0x8000)
#define GPIO_BIT_ALL                    ((uint16_t)0xFFFF)

#define IS_GPIO_WRITE(GPIO_x)                       (GPIO_SFRs[(GPIO_x)].PinCR)

#define IS_GPIO_BIT_DATA(GPIO_x,Bit_x)              ((((GPIO_SFRs[(GPIO_x)].PinDATA) & (Bit_x))&&\
                                                    (!((uint16_t)(~(GPIO_SFRs[(GPIO_x)].PinDATA))&(Bit_x)))))

#define IS_GPIO_BIT_OUT(GPIO_x,Bit_x)               (((GPIO_SFRs[(GPIO_x)].PinCR &(Bit_x))&&\
                                                    (!((uint16_t)(~GPIO_SFRs[(GPIO_x)].PinCR)&(Bit_x)))))

#define IS_GPIO_BIT_IN(GPIO_x,Bit_x)                (((GPIO_SFRs[(GPIO_x)].PinIE &(Bit_x))&&\
                                                    (!((uint16_t)(~GPIO_SFRs[(GPIO_x)].PinIE)&(Bit_x)))))

#define IS_GPIO_BIT_PUP(GPIO_x,Bit_x)               (((GPIO_SFRs[(GPIO_x)].PinPUP &(Bit_x))&&\
                                                    (!((uint16_t)(~GPIO_SFRs[(GPIO_x)].PinPUP)&(Bit_x)))))

#define IS_GPIO_BIT_PDN(GPIO_x,Bit_x)               (((GPIO_SFRs[(GPIO_x)].PinPDN &(Bit_x))&&\
                                                    (!((uint16_t)(~GPIO_SFRs[(GPIO_x)].PinPDN)&(Bit_x)))))

#define IS_GPIO_BIT_OD(GPIO_x,Bit_x)                (((GPIO_SFRs[(GPIO_x)].PinOD &(Bit_x))&&\
                                                    (!((uint16_t)(~GPIO_SFRs[(GPIO_x)].PinOD)&(Bit_x)))))

#define IS_GPIO_BIT_FR(GPIO_x,FuncReg_x,Bit_x)      (((GPIO_SFRs[(GPIO_x)].PinFR[(FuncReg_x)]&(Bit_x))&&\
                                                    (!((uint16_t)(~GPIO_SFRs[(GPIO_x)].PinFR[(FuncReg_x)])&(Bit_x)))))


#define IS_GPIO_FUNCTION_REG(param)     ((param) < (FRMAX))

#define IS_GPIO_BIT(param)              (((param) == GPIO_BIT_0)|| \
                                         ((param) == GPIO_BIT_1)|| \
                                         ((param) == GPIO_BIT_2)|| \
                                         ((param) == GPIO_BIT_3)|| \
                                         ((param) == GPIO_BIT_4)|| \
                                         ((param) == GPIO_BIT_5)|| \
                                         ((param) == GPIO_BIT_6)|| \
                                         ((param) == GPIO_BIT_7)|| \
                                         ((param) == GPIO_BIT_8)|| \
                                         ((param) == GPIO_BIT_9)|| \
                                         ((param) == GPIO_BIT_10)|| \
                                         ((param) == GPIO_BIT_11)|| \
                                         ((param) == GPIO_BIT_12)|| \
                                         ((param) == GPIO_BIT_13)|| \
                                         ((param) == GPIO_BIT_14)|| \
                                         ((param) == GPIO_BIT_15))
/** @} */
/* End of group GPIO_Bit_Define */

/** @defgroup GPIO_Exported_FunctionPrototypes
  * @{
  */
    uint16_t GPIO_ReadData(GPIO_Port GPIO_x);
    uint16_t GPIO_ReadDataBit(GPIO_Port GPIO_x, uint16_t Bit_x);
    void GPIO_WriteData(GPIO_Port GPIO_x, uint16_t Data);
    void GPIO_WriteDataBit(GPIO_Port GPIO_x, uint16_t Bit_x, uint16_t BitValue);
    void GPIO_Init(GPIO_Port GPIO_x, uint16_t Bit_x, GPIO_InitTypeDef * GPIO_InitStruct);
    void GPIO_SetOutput(GPIO_Port GPIO_x, uint16_t Bit_x);
    void GPIO_SetInput(GPIO_Port GPIO_x, uint16_t Bit_x);
    void GPIO_SetInputEnableReg(GPIO_Port GPIO_x, uint16_t Bit_x, FunctionalState NewState);
    void GPIO_SetOutputEnableReg(GPIO_Port GPIO_x, uint16_t Bit_x, FunctionalState NewState);
    void GPIO_SetPullUp(GPIO_Port GPIO_x, uint16_t Bit_x, FunctionalState NewState);
    void GPIO_SetPullDown(GPIO_Port GPIO_x, uint16_t Bit_x, FunctionalState NewState);
    void GPIO_SetOpenDrain(GPIO_Port GPIO_x, uint16_t Bit_x, FunctionalState NewState);
    void GPIO_EnableFuncReg(GPIO_Port GPIO_x, uint16_t FuncReg_x, uint16_t Bit_x);
    void GPIO_DisableFuncReg(GPIO_Port GPIO_x, uint16_t FuncReg_x, uint16_t Bit_x);
/** @} */
/* End of group GPIO_Exported_FunctionPrototypes */
/** @} */
/* End of group GPIO */
/** @} */
/* End of group TX04_Periph_Driver */


#ifdef __cplusplus
}
#endif                          /* __cplusplus */
#endif                          /* __TMPM461_GPIO_H */
