/** 
 *******************************************************************************
 * @file    cec.c
 * @brief   the application functions of CEC_TRX demo for the TOSHIBA
 *          'TMPM461' Device Series 
 * @version V0.101
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "cec.h"

/**
  * @brief  Initialize CEC register and variables
  * @param  None
  * @retval None
  */

void CEC_Init(void)
{

    CEC_Enable();
    CEC_SWReset();
    while (CEC_GetRxState() == ENABLE);
    while (CEC_GetTxState() == BUSY);

    CEC_DefaultConfig();
    CEC_SetLogicalAddr(CEC_TV);
    CEC_SetTxBroadcast(DISABLE);
    CEC_SetRxCtrl(ENABLE);

    gCEC_SendStatus.All = 0U;
    gCEC_SendMode = 0U;
    gCEC_RcvMode = 0U;
    gCEC_RcvCnt = 0U;
    gCEC_RcvEnd_Flag = 0U;

    gCEC_Command_Mode = CEC_CMD_MODE_IDLE;
}

/**
  * @brief  CEC STANDBY command send
  * @param  None
  * @retval None
  */
void CEC_Send_Standby(void)
{
    gCEC_SendDataBuf[0] = BROADCAST;    /* Broadcast mode       */
    gCEC_SendDataBuf[1] = OPSTANDBY;    /* CMD: STANDBY         */
    gCEC_SendByte = 0x01U;
    gCEC_SendCnt = 0U;
    gCEC_SendMode = 1U;
    gCEC_IO_Cectce.byte[0] = IP_CEC_BRD + IP_CEC_FREE_05;
    CEC_SetTxBroadcast(ENABLE);
    CEC_SetBusFreeTime(CEC_BUS_FREE_5_BIT);
    gCEC_IO_Cectce.byte[0] = IP_CEC_BRD + IP_CEC_FREE_03;
    gCEC_SendBuffer.byte[0] = gCEC_SendDataBuf[0];
    gCEC_SendBuffer.byte[1] = 0x00U;
    CEC_SetTxData(gCEC_SendBuffer.byte[0], CEC_NO_EOM);
    CEC_StartTx();
}

/**
  * @brief  CEC Remote Controll Through on TV
  * @param  reueset, key_value.
  * @retval None
  */
void CEC_RemoConThrough(uint8_t request, uint8_t key_value)
{
    switch (request) {
    case CMD_CEC_USER_CONTROL_PRESSED:
        gCEC_SendDataBuf[0] = RECDEVICE;        /* M461 -> DVR */
        gCEC_SendDataBuf[1] = CMD_CEC_USER_CONTROL_PRESSED;
        gCEC_SendDataBuf[2] = key_value;        /* UI Command Code */
        gCEC_SendByte = 0x02U;
        gCEC_SendCnt = 0U;
        gCEC_SendMode = 1U;
        gCEC_IO_Cectce.byte[0] = IP_CEC_FREE_05;
        CEC_SetTxBroadcast(DISABLE);
        CEC_SetBusFreeTime(CEC_BUS_FREE_5_BIT);
        gCEC_IO_Cectce.byte[0] = IP_CEC_FREE_03;
        gCEC_SendBuffer.byte[0] = gCEC_SendDataBuf[0];  /* Standby CMD */
        gCEC_SendBuffer.byte[1] = 0x00U;        /* EOF: The first data, not EOM. */
        CEC_SetTxData(gCEC_SendBuffer.byte[0], CEC_NO_EOM);
        CEC_StartTx();
        break;

    case CMD_CEC_USER_CONTROL_RELEASED:
        gCEC_SendDataBuf[0] = RECDEVICE;        /* M461 -> DVR */
        gCEC_SendDataBuf[1] = CMD_CEC_USER_CONTROL_RELEASED;
        gCEC_SendByte = 0x01U;
        gCEC_SendCnt = 0U;
        gCEC_SendMode = 1U;
        gCEC_IO_Cectce.byte[0] = IP_CEC_FREE_07;
        CEC_SetTxBroadcast(DISABLE);
        CEC_SetBusFreeTime(CEC_BUS_FREE_7_BIT);
        gCEC_IO_Cectce.byte[0] = IP_CEC_FREE_03;
        gCEC_SendBuffer.byte[0] = gCEC_SendDataBuf[0];
        gCEC_SendBuffer.byte[1] = 0x00U;        /* EOF: The first data, not EOM. */
        CEC_SetTxData(gCEC_SendBuffer.byte[0], CEC_NO_EOM);
        CEC_StartTx();
        break;

    default:
        /* do nothing */
        break;

    }
}

/**
  * @brief  CEC command send task
  * @param  None
  * @retval None
  */
void CEC_CMD_Task(void)
{
    switch (gCEC_Command_Mode) {
    case CEC_CMD_MODE_IDLE:
        if (CMD_Buffer.RdIdx < CMD_Buffer.WrIdx) {
            gCEC_Command_Mode = CEC_CMD_MODE_START;
        } else {
            /* do nothing */
        }
        break;

    case CEC_CMD_MODE_START:
        switch (CMD_Buffer.Data[CMD_Buffer.RdIdx]) {
        case KEY_POWER:        /* [Power]      */
            gCEC_Command_Mode = CEC_CMD_MODE_USER_PRESSED;
            gCEC_Send_Cmd = CEC_POWER;  /* 0x6b */
            break;

        case KEY_STANDBY:      /* [Standby]    */
            gCEC_Command_Mode = CEC_CMD_MODE_STANDBY_SEND;
            gCEC_Send_Cmd = CEC_STANDBY;
            break;

        case KEY_PLAY:         /* [Play]       */
            gCEC_Command_Mode = CEC_CMD_MODE_USER_PRESSED;
            gCEC_Send_Cmd = CEC_PLAY;   /* 0x60 */
            break;

        case KEY_STOP:         /* [Stop]       */
            gCEC_Command_Mode = CEC_CMD_MODE_USER_PRESSED;
            gCEC_Send_Cmd = CEC_STOP;   /* 0x64 */
            break;

        default:
            CMD_Buffer.RdIdx++;
            gCEC_Command_Mode = CEC_CMD_MODE_IDLE;
            break;
        }
        break;

    case CEC_CMD_MODE_STANDBY_SEND:
        CEC_Send_Standby();
        gCEC_Command_Mode = CEC_CMD_MODE_OK;
        break;

    case CEC_CMD_MODE_USER_PRESSED:
        CEC_RemoConThrough(CMD_CEC_USER_CONTROL_PRESSED, gCEC_Send_Cmd);
        gCEC_Command_Mode = CEC_CMD_MODE_USER_RELEASED;
        gCecTime = CECTIMEOUT;
        break;

    case CEC_CMD_MODE_USER_RELEASED:
        if (gCEC_SendMode == CLEAR) {   /* CEC Tx end: EOM        */
            if (gCecUnP == SET) {
                gCecUnP = CLEAR;

                CEC_RemoConThrough(CMD_CEC_USER_CONTROL_RELEASED, 0);
                gCEC_Command_Mode = CEC_CMD_MODE_OK;
            } else {
                /* do nothing */
            }
        } else {
            /* do nothing */
        }
        break;

    case CEC_CMD_MODE_OK:
        if (gCEC_SendMode == CLEAR) {   /* CEC Tx end: EOM      */
            gCEC_Command_Mode = CEC_CMD_MODE_IDLE;
            CMD_Buffer.RdIdx++;
        } else {
            /* do nothing */
        }
        break;

    default:
        /* do nothing */
        break;
    }
}

/**
  * @brief  CEC data Recive
  * @param  None
  * @retval None
  */
void CEC_Receive(void)
{
    if (gCEC_RcvMode == SET) {
        gCEC_RcvMode = CLEAR;
        gCEC_RcvDataBuf[gCEC_RcvCnt] = gCEC_RcvBuffer.Data;
        gCEC_RcvCnt++;
        if (gCEC_RcvBuffer.EOMBit == CEOF) {    /* EOF=1: End               */
            gCEC_RcvEnd_Flag = SET;     /* CEC frame received OK    */
        } else {
            /* do nothing */
        }
    } else {
        /* do nothing */
    }
}
