/** 
 *******************************************************************************
 * @file    cec.h
 * @brief   the application functions of CEC_TRX demo for the TOSHIBA
 *          'TMPM461' Device Series 
 * @version V0.101
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __CEC_H
#define __CEC_H
/* Includes ------------------------------------------------------------------*/
#include "main.h"

/* Private define ------------------------------------------------------------*/

#define CEOF            0x01U   /* EOF 1:end */
#define CECTIMEOUT      255U    /* CEC TIME OUT 255ms */
#define BROADCAST       0xefU   /* PHY:M461(14) LOGIC:BROADCAST(15) */
#define RECDEVICE       0x04U   /* PHY:M461(14) LOGIC:Recorder Device 2(2) */
#define OPSTANDBY       0x36U   /* OPCODE STANDBY */

#define CEC_CMD_MODE_IDLE               0x00U
#define CEC_CMD_MODE_START              0x01U

#define CEC_CMD_MODE_STANDBY_SEND       0x11U
#define CEC_CMD_MODE_USER_PRESSED       0x21U
#define CEC_CMD_MODE_USER_RELEASED      0x22U
#define CEC_CMD_MODE_OK                 0x30U


#define CMD_CEC_USER_CONTROL_PRESSED    0x44U
#define CMD_CEC_USER_CONTROL_RELEASED   0x45U

#define CEC_POWER                       0x40U
#define CEC_STANDBY                     0x36U
#define CEC_PLAY                        0x44U
#define CEC_STOP                        0x45U

#define IP_CEC_FREE_00		0x00000000U     /* Bus Free  1 bit     */
#define IP_CEC_FREE_01		0x00000001U     /* Bus Free  2 bit     */
#define IP_CEC_FREE_02		0x00000002U     /* Bus Free  3 bit     */
#define IP_CEC_FREE_03		0x00000003U     /* Bus Free  4 bit     */
#define IP_CEC_FREE_04		0x00000004U     /* Bus Free  5 bit     */
#define IP_CEC_FREE_05		0x00000005U     /* Bus Free  6 bit     */
#define IP_CEC_FREE_06		0x00000006U     /* Bus Free  7 bit     */
#define IP_CEC_FREE_07		0x00000007U     /* Bus Free  8 bit     */
#define IP_CEC_FREE_08		0x00000008U     /* Bus Free  9 bit     */
#define IP_CEC_FREE_09		0x00000009U     /* Bus Free 10 bit     */
#define IP_CEC_FREE_10		0x0000000AU     /* Bus Free 11 bit     */
#define IP_CEC_FREE_11		0x0000000BU     /* Bus Free 12 bit     */
#define IP_CEC_FREE_12		0x0000000CU     /* Bus Free 13 bit     */
#define IP_CEC_FREE_13		0x0000000DU     /* Bus Free 14 bit     */
#define IP_CEC_FREE_14		0x0000000EU     /* Bus Free 15 bit     */
#define IP_CEC_FREE_15		0x0000000FU     /* Bus Free 16 bit     */

#define IP_CEC_BRD		0x00000010U     /* CECBRD              */

/* globle variables ---------------------------------------------------------*/

CEC_DataTypeDef gCEC_RcvBuffer;
CEC_TxINTState gCEC_SendStatus;

IO_SFR16_u gCEC_SendBuffer;
IO_SFR32_u gCEC_IO_Cectce;

uint8_t gCEC_Command_Mode;

uint8_t gCEC_SendMode;
uint8_t gCEC_Send_Cmd;
uint8_t gCEC_SendDataBuf[17];
uint8_t gCEC_SendCnt;
uint8_t gCEC_SendByte;
uint8_t gCEC_SndEnd_Flag;

uint8_t gCEC_RcvDataBuf[17];
uint8_t gCEC_RcvCnt;
uint8_t gCEC_RcvMode;
uint8_t gCEC_RcvEnd_Flag;

uint8_t gCecTime;
uint8_t gCecUnP;

/* external variables --------------------------------------------------------*/
extern BUF_INFO CMD_Buffer;

/* Exported functions ------------------------------------------------------- */
void CEC_Init(void);
void CEC_Send_Standby(void);
void CEC_RemoConThrough(uint8_t request, uint8_t key_value);
void CEC_CMD_Task(void);
void CEC_Receive(void);
#endif                          /* CEC_TV_H */
/*********************************** END OF FILE ******************************/
