/**
 *******************************************************************************
 * @file    main.c
 * @brief   the application functions of CEC_TRX demo for the
 *          TOSHIBA 'TMPM461' Device Series
 * @version V0.101
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "main.h"

/* Globle variables ---------------------------------------------------------*/
BUF_INFO CMD_Buffer;

int char_PC2board;

#if defined ( __CC_ARM   )      /* RealView Compiler */
//struct __FILE { int handle;   /* Add whatever you need here */ };
// FILE __stdout;
extern FILE __stdin;
#endif

/**
  * @brief  main  function
  * @param  None
  * @retval None
  */

int main(void)
{

    uint8_t tmpCMD;
    uint8_t tmp_i;
    IO_SFR16_u RMCcmd;

    NVIC->ICER[0] = 0xffffffffU;
    NVIC->ICER[1] = 0xffffffffU;
    NVIC->ICPR[0] = 0xffffffffU;
    NVIC->ICPR[1] = 0xffffffffU;

    /*************** TIMER init *****************/

    Timer_ChInit(TSB_TB0);
    Timer_ChInit(TSB_TB1);

    /************* UART init **************/
    SIO_Configuration(UART0);
    SIO_ChInit();

    /************* RMC init **************/
    RMC_Configuration(TSB_RMC0);        /*RMC IO initial */
    RMC_ChInit(TSB_RMC0);       /* configure RMC0 */

    /************* CEC Port: PortK set **************/
    GPIO_SetOutputEnableReg(GPIO_PF, GPIO_BIT_13, ENABLE);
    GPIO_EnableFuncReg(GPIO_PF, GPIO_FUNC_REG_1, GPIO_BIT_13);
    GPIO_SetInputEnableReg(GPIO_PF, GPIO_BIT_13, ENABLE);

    /*************  CEC init **************/
    CEC_Init();

    /* set source(CECRX) to exit sleep mode */
    CG_SetSTBYReleaseINTSrc(CG_INT_SRC_CECRX, CG_INT_ACTIVE_STATE_RISING, ENABLE);

    /* set source(RMC0 interrupt) to exit sleep mode */
    CG_SetSTBYReleaseINTSrc(CG_INT_SRC_RMCRX0, CG_INT_ACTIVE_STATE_RISING, ENABLE);

    /* set source(CECTX) to exit sleep mode */
    CG_SetSTBYReleaseINTSrc(CG_INT_SRC_CECTX, CG_INT_ACTIVE_STATE_RISING, ENABLE);

    NVIC_EnableIRQ(INTRX0_IRQn);
    NVIC_EnableIRQ(INTTX0_IRQn);

    NVIC_EnableIRQ(INTCECRX_IRQn);
    NVIC_EnableIRQ(INTCECTX_IRQn);

    NVIC_EnableIRQ(INTRMCRX0_IRQn);

    NVIC_EnableIRQ(INTTB1_IRQn);
    TMRB_SetRunState(TSB_TB1, TMRB_RUN);

    printf("TMPM461 CEC DEMO\r\n");

    for (;;) {
        if (fRMCDone && (CMD_Buffer.WrIdx < CMD_BUFFER_SIZE)) {
            RMC_SetRxCtrl(TSB_RMC0, DISABLE);
            RMCcmd.reg = RMCRcv(TSB_RMC0);
#ifdef RMC_1
            tmpCMD = RMCcmd.byte[0];
#endif
#ifdef RMC_2
            tmpCMD = RMCcmd.byte[0] & 0x3FU;
#endif
#ifdef RMC_3
            tmpCMD = RMCcmd.byte[0];
#endif
#ifdef RMC_4
            tmpCMD = RMCcmd.byte[0];
#endif
#ifdef RMC_5
            tmpCMD = RMCcmd.byte[0];
#endif
            switch (tmpCMD) {
            case REMO_CTRL_1:  /* [1]      */
                CMD_Buffer.Data[CMD_Buffer.WrIdx++] = KEY_POWER;
                break;
            case REMO_CTRL_2:  /* [2]      */
                CMD_Buffer.Data[CMD_Buffer.WrIdx++] = KEY_STANDBY;
                break;
            case REMO_CTRL_3:  /* [3]      */
                CMD_Buffer.Data[CMD_Buffer.WrIdx++] = KEY_PLAY;
                break;
            case REMO_CTRL_4:  /* [4]      */
                CMD_Buffer.Data[CMD_Buffer.WrIdx++] = KEY_STOP;
                break;

            case REMO_CTRL_POWER:      /* [POWER]   */
                PowerMode_N2Stop1();
                break;

            default:
                /* do nothing */
                break;
            }
            RMC_SetRxCtrl(TSB_RMC0, ENABLE);
        } else {
            /* do nothing */
        }

        if (fSIORxOK == YES) {
#if defined ( __CC_ARM   )      /* RealView Compiler */
            char_PC2board = fgetc(stdin);
#elif defined ( __ICCARM__ )    /*IAR Compiler       */
            char_PC2board = getchar();
#endif
        } else {
            /* do nothing */
        }

        char_PC2board -= 0x30U;
        if ((char_PC2board >= 0x01U) && (char_PC2board <= 0x04U)) {
            CMD_Buffer.Data[CMD_Buffer.WrIdx++] = char_PC2board;
        } else {
            /* do nothing */
        }

        if (CMD_Buffer.RdIdx >= CMD_BUFFER_SIZE) {      /* reset buffer index */
            CMD_Buffer.WrIdx = 0U;
            CMD_Buffer.RdIdx = 0U;
        } else {
            /* do nothing */
        }

        CEC_CMD_Task();
        CEC_Receive();
        /* CEC Received data display on PC          */
        if (gCEC_RcvEnd_Flag == SET) {
            gCEC_RcvEnd_Flag = CLEAR;
            printf("CEC_Recv: ");
            for (tmp_i = 0U; tmp_i <= gCEC_RcvCnt; tmp_i++) {
                printf(" %02x", gCEC_RcvDataBuf[tmp_i]);
            }
            printf("\r\n");
            gCEC_RcvCnt = 0U;
        } else {
            /* do nothing */
        }

        /* CEC Send data display on PC          */
        if (gCEC_SndEnd_Flag == SET) {
            gCEC_SndEnd_Flag = CLEAR;
            printf("CEC_Send: ");
            for (tmp_i = 0U; tmp_i <= gCEC_SendCnt; tmp_i++) {
                printf(" %02x", gCEC_SendDataBuf[tmp_i]);
            }
            printf("\r\n");
            gCEC_SendCnt = 0U;
        } else {
            /* do nothing */
        }
    }
}

/**
  * @brief  Deal with the error parameter
  * @param  file: Pointer to the file where the error parameter locates
  * @param  line: Number of the line in which the error parameter locates
  * @retval None
  */
#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
