/** 
 *******************************************************************************
 * @file    main.h
 * @brief   the application functions of CEC_TRX demo for the TOSHIBA
 *          'TMPM461' Device Series 
 * @version V0.101
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __MAIN_H
#define __MAIN_H
/* Includes ------------------------------------------------------------------*/
#include "tmpm461_tmrb.h"
#include "tmpm461_uart.h"
#include "tmpm461_rmc.h"
#include "tmpm461_cg.h"
#include "tmpm461_gpio.h"
#include "tmpm461_cec.h"
#include "tmpm461_int.h"
#include "uart.h"
#include "timer.h"
#include "rmc.h"
#include "power_mode.h"
#include "led.h"
#include <stdio.h>

/* Private define ------------------------------------------------------------*/
#define CMD_BUFFER_SIZE   16U
#define SET              0x01U  /* flag is set */
#define CLEAR            0x00U  /* flag is cleared */

#ifdef RMC_1
/************** FOR RMC_1 FORMAT ****************/
#define REMO_CTRL_1         0x80U
#define REMO_CTRL_2         0x40U
#define REMO_CTRL_3         0xc0U
#define REMO_CTRL_4         0x20U
#define REMO_CTRL_POWER     0x48U
#endif

#ifdef RMC_2
/************** FOR RMC_2 FORMAT ****************/
#define REMO_CTRL_1         0x01U
#define REMO_CTRL_2         0x02U
#define REMO_CTRL_3         0x03U
#define REMO_CTRL_4         0x04U
#define REMO_CTRL_POWER     0x0cU
#endif

#ifdef RMC_3
/************** FOR RMC_3 FORMAT ****************/
#define REMO_CTRL_1         0x09U
#define REMO_CTRL_2         0x89U
#define REMO_CTRL_3         0x49U
#define REMO_CTRL_4         0xc9U
#define REMO_CTRL_POWER     0xbdU
#endif

#ifdef RMC_4
/************** FOR RMC_4 FORMAT ****************/
#define REMO_CTRL_1         0x00U
#define REMO_CTRL_2         0x40U
#define REMO_CTRL_3         0x20U
#define REMO_CTRL_4         0x60U
#define REMO_CTRL_POWER     0x54U
#endif

#ifdef RMC_5
/************** FOR RMC_5 FORMAT ****************/
#define REMO_CTRL_1         0x80U
#define REMO_CTRL_2         0x40U
#define REMO_CTRL_3         0xc0U
#define REMO_CTRL_4         0x20U
#define REMO_CTRL_POWER     0x68U
#endif

#define KEY_POWER           0x01U
#define KEY_STANDBY         0x02U
#define KEY_PLAY            0x03U
#define KEY_STOP            0x04U

//#if defined ( __CC_ARM   )    /* RealView Compiler */
//struct __FILE { int handle;   /* Add whatever you need here */ };
//FILE __stdout;
//FILE __stdin;
//#endif

/* Private typedef -----------------------------------------------------------*/
typedef struct {
    uint8_t WrIdx;              /* Next In Index */
    uint8_t RdIdx;              /* Next Out Index */
    uint8_t Data[CMD_BUFFER_SIZE];      /* Buffer */
} BUF_INFO;

typedef union tag_IO_SFR16_u {
    uint16_t reg;
    uint8_t byte[2];
} IO_SFR16_u;

typedef union tag_IO_SFR32_u {
    uint32_t reg;
    uint8_t byte[4];
} IO_SFR32_u;

/* external variables --------------------------------------------------------*/
extern uint8_t gCEC_RcvMode;
extern uint8_t gCEC_RcvCnt;
extern uint8_t gCEC_RcvEnd_Flag;
extern uint8_t gCEC_SndEnd_Flag;
extern uint8_t gCEC_SendCnt;
extern uint8_t gCEC_RcvDataBuf[];
extern uint8_t gCEC_SendDataBuf[];

/* External function prototypes ----------------------------------------------*/
extern uint16_t RMCRcv(TSB_RMC_TypeDef * RmcChSel);
extern void PowerMode_N2Stop1(void);
extern void CEC_CMD_Task(void);
extern void CEC_Receive(void);
extern void CEC_Init(void);     /* Integrate CEC */

#endif
