/** 
 *******************************************************************************
 * @file    timer.c
 * @brief   the application functions of RMC_Receiving demo for the TOSHIBA
 *          'TMPM461' Device Series 
 * @version V0.101
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "timer.h"
uint16_t Timer_Calculator(uint16_t Tmrb_Require_us, uint32_t ClkDiv) ;

/**
  * @brief  Initialize the timer.
  * @param  TMRB channel
  * @retval None
  */

void Timer_ChInit(TSB_TB_TypeDef * TBx)
{
    TMRB_InitTypeDef myTMRB;

    /* Initialize TIMER(TMRB) */
    myTMRB.Mode = TMRB_INTERVAL_TIMER;
    myTMRB.ClkDiv = TMRB_CLK_DIV_8;
    /* periodic time is 1ms(require 1000us) */ 
    myTMRB.TrailingTiming = Timer_Calculator(1000U, myTMRB.ClkDiv); 
    myTMRB.UpCntCtrl = TMRB_AUTO_CLEAR;
    /* periodic time is 1ms(require 1000us) */
    myTMRB.LeadingTiming = Timer_Calculator(1000U, myTMRB.ClkDiv);
    
    TMRB_Enable(TBx);
    TMRB_Init(TBx, &myTMRB);
    TMRB_SetRunState(TBx, TMRB_STOP);

}

/** 
  * @brief  Calculate timer value to set timer register. 
  * @param  Tmrb_Require_us: The require period which the uint is us. 
            ClkDiv: Select the division for TMRB source clock 
  * @retval the value set to Tmrb timer register. 
  */ 
uint16_t Timer_Calculator(uint16_t Tmrb_Require_us, uint32_t ClkDiv)
{      
    uint32_t T0 = 0U; 
    const uint16_t Div[8U] = {1U, 2U, 8U, 32U, 64U, 128U, 256U, 512U}; 
    
    SystemCoreClockUpdate(); 

    T0 = SystemCoreClock / (1U << ((TSB_CG->SYSCR >> 8U) & 7U));   
    T0 = T0/((Div[ClkDiv])*1000000U); 
         
    return(Tmrb_Require_us * T0);

} 
