/** 
 *******************************************************************************
 * @file    uart.c
 * @brief   the application functions of UART_Retarget demo for the TOSHIBA
 *          'TMPM461' Device Series 
 * @version V0.101
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "uart.h"

/* globle variables ---------------------------------------------------------*/
char gSIOTxBuffer[BUFFER_SIZE] = { 0 };
char gSIORxBuffer[8] = { 0 };

uint8_t gSIOWrIndex = 0U;
uint8_t gSIORdIndex = 0U;
uint8_t fSIO0_INT = 0U;
uint8_t fSIORxOK = NO;

/**
  * @brief  SIO Configuration.
  * @param  None
  * @retval None
  */

void SIO_Configuration(TSB_SC_TypeDef * SCx)
{
    if (SCx == UART0) {
        GPIO_SetOutputEnableReg(GPIO_PF, GPIO_BIT_3, ENABLE);
        GPIO_EnableFuncReg(GPIO_PF, GPIO_FUNC_REG_1, GPIO_BIT_3);
        GPIO_EnableFuncReg(GPIO_PF, GPIO_FUNC_REG_1, GPIO_BIT_4);
        GPIO_SetInputEnableReg(GPIO_PF, GPIO_BIT_4, ENABLE);
    } else if (SCx == UART1) {
        GPIO_SetOutputEnableReg(GPIO_PF, GPIO_BIT_6, ENABLE);
        GPIO_EnableFuncReg(GPIO_PF, GPIO_FUNC_REG_1, GPIO_BIT_6);;
        GPIO_EnableFuncReg(GPIO_PF, GPIO_FUNC_REG_1, GPIO_BIT_7);;
        GPIO_SetInputEnableReg(GPIO_PF, GPIO_BIT_7, ENABLE);
    } else if (SCx == UART2) {
        GPIO_SetOutputEnableReg(GPIO_PF, GPIO_BIT_9, ENABLE);
        GPIO_EnableFuncReg(GPIO_PF, GPIO_FUNC_REG_1, GPIO_BIT_9);
        GPIO_EnableFuncReg(GPIO_PF, GPIO_FUNC_REG_1, GPIO_BIT_10);
        GPIO_SetInputEnableReg(GPIO_PF, GPIO_BIT_10, ENABLE);
    } else {
        /* Do nothing */
    }
}

/**
  * @brief  SIO Configuration Initialize the uart port (SIO0).
  * @param  None
  * @retval None
  */

void SIO_ChInit(void)
{
    UART_InitTypeDef myUART;

    /* configure SIO0 for reception */
    UART_Enable(UART0);
    myUART.BaudRate = 115200U;  /* baut rate = 115200 */
    myUART.DataBits = UART_DATA_BITS_8; /* no handshake, 8-bit data, clock by baud rate generator */
    myUART.StopBits = UART_STOP_BITS_1; /* 1-bit stop, LSB,  W-buff enable */
    myUART.Parity = UART_NO_PARITY;
    myUART.Mode = UART_ENABLE_RX | UART_ENABLE_TX;
    myUART.FlowCtrl = UART_NONE_FLOW_CTRL;
    UART_Init(UART0, &myUART);

}

/**
  * @brief  Send character by SIO0
  * @param  Character to be sent
  * @retval Character to be sent
  */
uint8_t send_char(uint8_t ch)
{
    while (gSIORdIndex != gSIOWrIndex) {        /* wait for finishing sending */
        /* do nothing */
    }
    gSIOTxBuffer[gSIOWrIndex++] = ch;   /* fill TxBuffer */
    if (fSIO0_INT == CLEAR) {   /* if SIO0 INT disable, enable it */
        fSIO0_INT = SET;        /* set SIO0 INT flag */
        UART_SetTxData(UART0, gSIOTxBuffer[gSIORdIndex++]);
        NVIC_EnableIRQ(INTTX0_IRQn);
        NVIC_EnableIRQ(INTRX0_IRQn);
        NVIC_EnableIRQ(INTCECRX_IRQn);
        NVIC_EnableIRQ(INTCECTX_IRQn);
    }

    return ch;
}
