/** 
 *******************************************************************************
 * @file    rmc.h
 * @brief   the application functions of RMC_Receiving demo for the TOSHIBA
 *          'TMPM461' Device Series 
 * @version V0.101
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __RMC_H
#define __RMC_H

/* Includes ------------------------------------------------------------------*/
#include "main.h"

/* Private typedef -----------------------------------------------------------*/
typedef enum { RMC_NON = 0, RMC_DONE = 1 } RMCSTAT;
typedef enum { RELEASE = 0, PRESS = 1 } RMCPRS;
typedef union tag_IO_SFR32_u {
    uint32_t reg;
    uint8_t byte[4];
} IO_SFR32_u;

/* Private define ------------------------------------------------------------*/
/**************   CHOOSE ONE FORMAT   **************/
/*  #define RMC_1 */
/*  #define RMC_2 */
/*  #define RMC_3 */
/*  #define RMC_4 */
/*  #define RMC_5 */
#define RMC_6

#ifdef RMC_1                    /*TOSHIBA format */
/************** FOR RMC_1 FORMAT ****************/
#define RMC_MAX_CYCLE                   ((uint8_t)0x71)
#define RMC_MIN_CYCLE                   ((uint8_t)0x5A)
#define RMC_MAX_LOW_WIDTH               ((uint8_t)0x4C)
#define RMC_MIN_LOW_WIDTH               ((uint8_t)0x49)

#define RMC_TRG_LOW_WIDTH               ((uint8_t)0xFF)
#define RMC_TRG_MAX_DATA_BIT_CYCLE      ((uint8_t)0xFE)

#define RMC_LARGER_THRESHOLD            ((uint8_t)0x00)
#define RMC_SMALLER_THRESHOLD           ((uint8_t)0x2E)

#define RMC_NOISE_CANCELLATION_TIME     ((uint8_t)0x0D)
#endif

#ifdef RMC_2                    /*RC5 */
/************** FOR RMC_2 FORMAT ****************/
/* no leader */
#define RMC_MAX_CYCLE                   ((uint8_t)0x00)
#define RMC_MIN_CYCLE                   ((uint8_t)0x00)
#define RMC_MAX_LOW_WIDTH               ((uint8_t)0x00)
#define RMC_MIN_LOW_WIDTH               ((uint8_t)0x00)

#define RMC_TRG_LOW_WIDTH               ((uint8_t)0x47)
#define RMC_TRG_MAX_DATA_BIT_CYCLE      ((uint8_t)0x83)

#define RMC_LARGER_THRESHOLD            ((uint8_t)0x63)
#define RMC_SMALLER_THRESHOLD           ((uint8_t)0x44)

#define RMC_NOISE_CANCELLATION_TIME     ((uint8_t)0x0F)

#endif

#ifdef RMC_3                    /*uf */
/************** FOR RMC_3 FORMAT ****************/
#define RMC_MAX_CYCLE                   ((uint8_t)0x21)
#define RMC_MIN_CYCLE                   ((uint8_t)0x17)
#define RMC_MAX_LOW_WIDTH               ((uint8_t)0x11)
#define RMC_MIN_LOW_WIDTH               ((uint8_t)0x0B)

#define RMC_TRG_LOW_WIDTH               ((uint8_t)0xFF)
#define RMC_TRG_MAX_DATA_BIT_CYCLE      ((uint8_t)0x62)

#define RMC_LARGER_THRESHOLD            ((uint8_t)0x00)
#define RMC_SMALLER_THRESHOLD           ((uint8_t)0x27)

#define RMC_NOISE_CANCELLATION_TIME     ((uint8_t)0x00)
#endif

#ifdef RMC_4 /*SONY*/
/************** FOR RMC_4 FORMAT ****************/
#define RMC_MAX_CYCLE                   ((uint8_t)0x19)
#define RMC_MIN_CYCLE                   ((uint8_t)0x0F)
#define RMC_MAX_LOW_WIDTH               ((uint8_t)0x00)
#define RMC_MIN_LOW_WIDTH               ((uint8_t)0x00)
#define RMC_TRG_LOW_WIDTH               ((uint8_t)0xFF)
#define RMC_TRG_MAX_DATA_BIT_CYCLE      ((uint8_t)0x85)
#define RMC_LARGER_THRESHOLD            ((uint8_t)0x00)
#define RMC_SMALLER_THRESHOLD           ((uint8_t)0x31)
#define RMC_NOISE_CANCELLATION_TIME     ((uint8_t)0x0D)
#endif
#ifdef RMC_5 /*SHARP*/
/************** FOR RMC_5 FORMAT ****************/
/* no leader */
#define RMC_MAX_CYCLE                   ((uint8_t)0x00)
#define RMC_MIN_CYCLE                   ((uint8_t)0x00)
#define RMC_MAX_LOW_WIDTH               ((uint8_t)0x00)
#define RMC_MIN_LOW_WIDTH               ((uint8_t)0x00)
#define RMC_TRG_LOW_WIDTH               ((uint8_t)0xFF)
#define RMC_TRG_MAX_DATA_BIT_CYCLE      ((uint8_t)0x85)
#define RMC_LARGER_THRESHOLD            ((uint8_t)0x00)
#define RMC_SMALLER_THRESHOLD           ((uint8_t)0x32)
#define RMC_NOISE_CANCELLATION_TIME     ((uint8_t)0x03)
#endif
#ifdef RMC_6 /*NEC*/
/************** FOR RMC_6 FORMAT ****************/
#define RMC_MAX_CYCLE                   ((uint8_t)0x52)
#define RMC_MIN_CYCLE                   ((uint8_t)0x42)
#define RMC_MAX_LOW_WIDTH               ((uint8_t)0x29)
#define RMC_MIN_LOW_WIDTH               ((uint8_t)0x21)
#define RMC_TRG_LOW_WIDTH               ((uint8_t)0xFF)
#define RMC_TRG_MAX_DATA_BIT_CYCLE      ((uint8_t)0xFE)
#define RMC_LARGER_THRESHOLD            ((uint8_t)0x00)
#define RMC_SMALLER_THRESHOLD           ((uint8_t)0x25)
#define RMC_NOISE_CANCELLATION_TIME     ((uint8_t)0x03)
#endif
/* external variables --------------------------------------------------------*/
extern RMCSTAT fRMCDone;
extern RMCPRS fRMCPress;
/* Private function prototypes -----------------------------------------------*/
void RMC_Configuration(TSB_RMC_TypeDef * RMCx);
void RMC_ChInit(TSB_RMC_TypeDef * RMCx);
void RMCRcv(void);

#endif                          /* __RMC_H */
